import bpy
import math
import mathutils
from mathutils import Vector
from math import degrees

# to load script from editor as a module in Python console use the following: m = bpy.data.texts['Text'].as_module()
# then you can call the module functions as e.g.: m.add_line(id_from, id_to, material, text)
# test it by: m.add_line('mother', 'child', 'Edge_material', '  some_text')

def select_set_active(object):
    bpy.ops.object.select_all(action='DESELECT')
    object.select_set(True)      
    bpy.context.view_layer.objects.active = object


def modify_line_objs(obj, emptyObj, textObj, mother, child, radius, text):
    visibility = obj.hide_viewport # get original visibility
    obj.hide_viewport = False # make object visible, otherwise transformations can not be applied
    emptyObj.hide_viewport = False # make object visible, otherwise transformations can not be applied
    textObj.hide_viewport = False # make object visible, otherwise transformations can not be applied

    toCoord = child.location
    fromCoord = mother.location
    direction = toCoord - fromCoord
    obj.scale = (radius, radius, direction.length/2)
    obj.rotation_euler.zero()
    initial_direction = Vector((0,1,0))
    rot = initial_direction.rotation_difference(direction).to_euler()
    
    # setup empty as a text holder
    emptyObj.name = obj.name + "_empty"
    # setup text
    textObj.name = obj.name + "_text"
    textObj.data.body = text
    textObj.scale = (15,15,15)
    textObj.rotation_euler[0] = 1.5708
    textObj.rotation_euler[2] = 3.0*1.5708

    # rotate around x to make Y-axis the main
    select_set_active(obj)
    obj.rotation_euler.x = -math.pi/2
    bpy.ops.object.transform_apply(location=False, rotation=True, scale=True) # make the rotation transform permanent

    # move origin
    obj.data.transform(mathutils.Matrix.Translation(-Vector((0, -direction.length/2, 0))))
    obj.matrix_world.translation += Vector((0, -direction.length/2, 0))        
    
    
    # constraint text to empty
    select_set_active(textObj)
    bpy.ops.object.constraint_add(type='COPY_LOCATION')
    bpy.context.object.constraints["Copy Location"].target = emptyObj

    # assign empty to obj
    bpy.ops.object.select_all(action='DESELECT')
    emptyObj.select_set(True)
    obj.select_set(True)
    bpy.context.view_layer.objects.active = obj
    bpy.ops.object.parent_set(type='OBJECT', keep_transform=False)    

    # assign label text to the empty
    textObj.parent = emptyObj


    # translate and rotate between mother-child
    obj.location = fromCoord
    obj.rotation_euler = rot
    
    # create final relations of objects
    bpy.ops.object.constraint_add(type='STRETCH_TO')
    bpy.context.object.constraints["Stretch To"].target = child
    bpy.context.object.constraints["Stretch To"].bulge = 0
    bpy.ops.object.select_all(action='DESELECT')
    obj.select_set(True)
    child.select_set(True)
    mother.select_set(True)      
    bpy.context.view_layer.objects.active = mother
    bpy.ops.object.parent_set(type='OBJECT', keep_transform=False)
    bpy.ops.object.select_all(action='DESELECT')

    obj.hide_viewport = visibility # revert object visibility to original one
    emptyObj.hide_viewport = visibility # revert object visibility to original one
    textObj.hide_viewport = visibility # revert object visibility to original one
    

